/*
 * File : dlReject.C
 *
 * Description : This file contains the method definitions for
 * the dlReject class.
 */

#include "types.h"
#include "frame.h"
#include "dlEvents.h"
#include "dlInfoXfer.h"
#include "dlReceiverBusy.h"
#include "dlReject.h"
#include "dlDisconnected.h"
#include "dllayer.h"

// the receiverBusy variable tells us whether we should
// should go into busy state or not. It is set in the nwlayer/user
// interface. The user can toggle the value of this variable
// from TRUE to FALSE and vice-versa.

extern Boolean receiverBusy;

/*
 * Function : dlReject()
 *
 * Description : This is the constructor. It invokes the
 *	base class constructor. Note the second argument of
 * the dlInfoXfer constructor is FALSE. This means that
 * the state variables of the data-link are not to be
 * reset.
 */

dlReject::dlReject(dllayer *dl) : dlInfoXfer(dl,FALSE) 
{
}

/*
 * Function : PrintState()
 *
 * Description : Prints the state.
 */

void 
dlReject::PrintState() 
{
	TRACE(dlSmModule,1,("State dlReject\n"));
}

/*
 * Function : HandleIframe()
 *
 * Description : Processes an I-frame that was received.
 * First, check validity of N(r). If invalid, invoke
 * HandleFrmrCondition() with Z bit set to 1.
 * Next check for the expected N(s) sequence number in
 * the I-frame. Discard I-frame if invalid and return.
 * If N(s) is valid, accept it, clear REJ condition by
 * transmitting a RR. Invoke DL_DataInd on network layer.
 * Check receiverBusy flag to check if DTE should go 
 * into busy state or back to normal information transfer 
 * state.
 */


dlStateMachine *
dlReject::HandleIframe(int &errorCode)
{
	_dl->_lastFrameRecd->PrintParams();

	// Check for ack
	// Transmit FRMR if invalid N(r)

	int nr = _dl->_lastFrameRecd->GetNr();
	int numAcked;
	if ( (numAcked = _dl->_win.ProcessAck(nr)) == -1 )
		return HandleFrmrCondition(Z,errorCode);

	unsigned int ns = _dl->_lastFrameRecd->GetNs();

	// Discard out-of-sequence iframe
	if ( _dl->_win.CheckSequence(ns) == FALSE ) {
		TRACE(dlSmModule,1,("Out of sequence frame\n"));
		return this;
	}

	// Acknowledge the Iframe, and clear the Reject condition
	_dl->XmitFrame(errorCode,FALSE,RR,_dl->_lastFrameRecd->GetPf(),
		_dl->_win.GetVr());


	// Notify the network layer
	_dl->_nPlus1->Command(DL_DataInd,errorCode,
		_dl->_lastFrameRecd->GetDataPacket());


	// Check if receiver is busy or not!
	if ( receiverBusy == FALSE ) {
		CHANGE_TO_INFOXFER(FALSE);
	}

	// Receiver is busy ...change back to dlReceiverBusy
	CHANGE_STATE(dlReceiverBusy);
}

/*
 * Function : GetSupervisoryFrame()
 *
 * Description : Returns RNR for the state of the DTE.
 */

int 
dlReject::GetSupervisoryFrame()
{
	return REJ;
}


dlStateMachine *
dlReject::HandleRejTimer(int &errorCode)
{
	TRACE(dlSmModule,1,("In HandleRejTimer!\n"));

	if ( _dl->_rejRetryCount < N2 ) {
		_dl->XmitFrame(errorCode,TRUE,REJ,1,_dl->_win.GetVr());

		// Restart REJ Timer
		_dl->StartRejTimer(TRUE);
		return this;
	}
	else { 
		TRACE(dlSmModule,1,("Rej Timer timed out N2 times\n"));
		_dl->_rejTimer = 0;
		return ( LinkReset(errorCode));
	}
}
