/*
 * File : dllayer.h
 *
 * Description : This file contains the class declaration for dllayer.
 */

#ifndef _dllayer_h_
#define _dllayer_h_


#include "trace.h"
#include "layer.h"
#include "errorCodes.h"
#include "window.h"

/*
 * enum to differentiate between different types of reset.
 */

enum setupType { initial=0, externalReset, internalReset };

class frame;
class dlStateMachine;
class timerT1;

/*
 * Class : dllayer
 *
 * Description : dllayer class provides the functionality to
 * implement the data-link layer. It inherits from layer class
 * to provide a standard layer interface.
 *
 * Attributes:
 *	dlStateMachine *_sm : Pointer to the current state machine
 *			object.
 *	timerT1 *_t1Timer : Pointer to a T1 timer which may be active.
 *	timert1 *_rejTimer: Pointer to a REJ timer which may be active.
 *
 *	int	_t1RetryCount : The retry count associated with the T1 timer.
 *	int	_rejRetryCount: The retry count associated with the REJ timer.
 *
 *	Boolean	_remoteBusy: Flag, if TRUE remote DTE is busy.
 *
 *	frame *_lastFrameRecd: Pointer to the last frame received.
 *	int	_expFBit: If set to 1, we are expecting a response with F=1.
 *	setupType _setupType: The setup type for a pending link setup/reset.
 *	window	_win: The window for this data-link layer.
 *
 *	unsigned char _frmrInfo[3]: The FRMR info field for the last FRMR
 *		sent.
 */

class dllayer : public layer {

	// All state machine classes are friends since
	// they need to access the private data of dllayer

	friend class dlStateMachine;
	friend class dlDisconnected;
	friend class dlInSetupPending;
	friend class dlOutSetupPending;
	friend class dlInfoXfer;
	friend class dlDiscPending;
	friend class dlFrameReject;
	friend class dlReceiverBusy;
	friend class dlReject;
	friend class timerT1;

 public:

	dllayer();
	virtual ~dllayer();

	virtual void Command(int cmd, int &errorCode);
	virtual void Command(int cmd, int &errorCode, dataPacket *data);
	virtual void Command(int cmd, int &errorCode, int reason);
	int Init();

 private:
	void StartT1Timer(Boolean continueRetry);
	void StartRejTimer(Boolean continueRetry);

	void StopT1Timer();
	void StopRejTimer();

	timerT1 * StartTimer(int event); 

	void ResetStateVariables();

	dlStateMachine *_sm;
	void	ProcessEvent(int event, int &errorCode);

	timerT1 *_t1Timer;
	timerT1 *_rejTimer;
	int		_t1RetryCount;
	int		_rejRetryCount;
	Boolean	_remoteBusy;

	frame *_lastFrameRecd;
	int	_expFBit;
	setupType _setupType;
	window	_win;

	unsigned char _frmrInfo[3];
	void	XmitFrmr(int pf,int &errorCode);
	void	XmitFrame(int &errorCode, Boolean cmd, int frameType, int pf, 
				int nr = 0, int ns = 0);  
};

DECLARE_TRACE_MODULE(dllayerModule);

#endif
